package com.example.youtube_videos.activity;

import java.util.logging.Logger;

import com.example.youtube_videos.GetUserDetailAsyncTask;
import com.example.youtube_videos.IDataAccessor;
import com.example.youtube_videos.R;
import com.example.youtube_videos.R.id;
import com.example.youtube_videos.R.layout;
import com.example.youtube_videos.R.menu;
import com.example.youtube_videos.fragment.AlertDialogFragment;
import com.example.youtube_videos.fragment.RetainedWorkerFragment;
import com.example.youtube_videos.model.UserDetail;
import com.example.youtube_videos.util.Settings;
import com.example.youtube_videos.util.Util;

import android.os.Bundle;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class MainActivity extends FragmentActivity implements IDataAccessor {

	private Logger log = Logger.getLogger(MainActivity.class.getName());
	
	public static final String LOG_TAG = "yt-video";
	
	private static final int REQUEST_USER = 1;
	

	
	private RetainedWorkerFragment mWorkFragment;
	private EditText etUser;
	private TextView tvTitle;
	private TextView tvDescription;
	
	private Settings settings;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        settings = new Settings(this);
        
        etUser = (EditText) findViewById(R.id.et_user);
        etUser.setText(settings.getUser());
        
        // go to UserListActivity to select the user
        // we want a result to be returned from UserListActivity
        // no data are passed to UserListActivity
        Button btnGetUser = (Button) findViewById(R.id.btn_get_user);
        btnGetUser.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				removeFragment();
				
				Intent intent = new Intent(MainActivity.this, UserListActivity.class);
				startActivityForResult(intent, REQUEST_USER);
			}
		});
        
        
        
        tvTitle = (TextView) findViewById(R.id.txt_title);
        tvDescription = (TextView) findViewById(R.id.txt_description);
        
        
        // Check to see if we have retained the worker fragment.
        final FragmentManager fm = getSupportFragmentManager();
        mWorkFragment = (RetainedWorkerFragment) fm.findFragmentByTag("work");
        
        // get detail - wrong implementation (although it might run sometimes)
        // DON'T use networking on the main thread!!!
        Button btnGetDetail = (Button) findViewById(R.id.btn_get_detail);
        btnGetDetail.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				UserDetail ud = null;
				try {
					String response = Util.downloadUserDetail(etUser.getText().toString());
					ud = Util.parseUserDetail(response);
				} catch(Exception e) {
					log.severe("An exception occurred: " + e.getMessage());
					
					// or another way to create logs
					Log.e(LOG_TAG, "An exception occurred: " + e.getMessage());
				}
				setUserDetail(ud);
			}
		});
        
        // better solution - download moved to worker thread implemented via AsyncTask
        // one problem here - if the activity dies (e.g. config change), the async processing is no longer attached to the activity
        // we need to reconnect the async processing to the new activity when it is recreated
        Button btnGetDetailBetter = (Button) findViewById(R.id.btn_get_detail_better);
        btnGetDetailBetter.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				new GetUserDetailAsyncTask(MainActivity.this).execute(etUser.getText().toString());
			}
		});
        
        // perfect solution - async processing reconnected when the activity is recreated
        Button btnGetDetailBest = (Button) findViewById(R.id.btn_get_detail_best);
        btnGetDetailBest.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				mWorkFragment = RetainedWorkerFragment.newInstance(MainActivity.this, etUser.getText().toString());
				fm.beginTransaction().add(mWorkFragment, "work").commit();
			}
		});

        // display the user's videos in the VideoListActivity
        // we are passing data to the VideoListActivity
        Button btnGetVideos = (Button) findViewById(R.id.btn_get_videos);
        btnGetVideos.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				removeFragment();
				
				Intent intent = new Intent(MainActivity.this, VideoListActivity.class);
				intent.putExtra(VideoListActivity.EXTRA_USER, etUser.getText().toString());
				startActivity(intent);
			}
		});
        
        Button btnClearDetail = (Button) findViewById(R.id.btn_clear_detail);
        btnClearDetail.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				tvTitle.setText("");
				tvDescription.setText("");
			}
		});
         
    }
    
    private void removeFragment() {
    	if(mWorkFragment != null) {
    		FragmentManager fm = getSupportFragmentManager();
    		fm.beginTransaction().remove(mWorkFragment).commit();
    		mWorkFragment = null;
    	}
    }

	public void setUserDetail(UserDetail ud) {
		if(ud != null) {
			tvTitle.setText(ud.getTitle());
			tvDescription.setText(ud.getDescription());
		}
	}

	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent) {
		// process the results returned from other activities
		if(requestCode == REQUEST_USER) {
			if(resultCode == RESULT_OK) {
				String user = intent.getExtras().getString(UserListActivity.EXTRA_USER);
				etUser.setText(user);
				settings.saveUser(user);
				return;
			}
		}
		super.onActivityResult(requestCode, resultCode, intent);
	}

	

    

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	// create menu
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		// menu item selected, handle some action
		if(item.getItemId() == R.id.menu_dialog) {
			// display simple alert dialog
			AlertDialogFragment fragment = AlertDialogFragment.newInstance(etUser.getText().toString());
			FragmentManager fm = getSupportFragmentManager();
			fm.beginTransaction().add(fragment, "dialog").commit();
			return true;
		}
		return super.onMenuItemSelected(featureId, item);
	}

}
